package at.ac.uibk.gitsearch.web.rest;

import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.io.IOException;

import org.elasticsearch.node.NodeValidationException;
import org.junit.Assert;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultMatcher;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.repository.search.testESService.ElasticSearchTestServerConfiguration;
import at.ac.uibk.gitsearch.security.AuthoritiesConstants;

/**
 * Integration tests for the {@link ExerciseResource} REST controller.
 */
@AutoConfigureMockMvc
@WithMockUser(authorities = AuthoritiesConstants.USER)
@SpringBootTest(classes = GitsearchApp.class)
public class ExerciseResourceIT {

	@BeforeAll
	public static void setUpESServer() throws IOException, NodeValidationException {
		ElasticSearchTestServerConfiguration.getTestInstance().startTestNode();
	}

    @Autowired
    private MockMvc restExerciseMockMvc;



    @Test
    public void getREADME() throws Exception {
    	String exerciseId = "272"; // warning: this depends on the sequence in the current search index :-(
    	restExerciseMockMvc.perform(get("/api/exerciseFile/"+exerciseId).param("filePath", "README.md")
            	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON))
//    		.andDo(print())
            .andExpect(status().is2xxSuccessful())
            .andExpect(StreamResultMatcher.expectOctetStream())
            .andExpect(StreamResultMatcher.expectMinimumSize(50));

    }


    @Test
    public void getExercisemd() throws Exception {
    	String exerciseId = "272"; // warning: this depends on the sequence in the current search index :-(
    	restExerciseMockMvc.perform(get("/api/exerciseFile/"+exerciseId).param("filePath", "exercise.md")
            	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON))
    		.andDo(print())
    		.andExpect(status().is2xxSuccessful())
            .andExpect(StreamResultMatcher.expectOctetStream())
            .andExpect(StreamResultMatcher.expectMinimumSize(50));

    }
    
    @Test
    public void getNestedPath() throws Exception {
    	String exerciseId = "272"; // warning: this depends on the sequence in the current search index :-(
    	restExerciseMockMvc.perform(get("/api/exerciseFile/"+exerciseId).param("filePath", "solution/src/at/ac/uibk/ac/artemis/Receiver.java")
            	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON))
    		.andDo(print())
    		.andExpect(status().is2xxSuccessful())
            .andExpect(StreamResultMatcher.expectOctetStream())
            .andExpect(StreamResultMatcher.expectMinimumSize(50));
    }

    @Test
    public void getSplitPath() throws Exception {
    	String exerciseId = "272:solution/src/at"; // warning: this depends on the sequence in the current search index :-(
    	restExerciseMockMvc.perform(get("/api/exerciseFile/"+exerciseId).param("filePath", "ac/uibk/ac/artemis/Receiver.java")
            	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON))
    		.andDo(print())
    		.andExpect(status().is2xxSuccessful())
            .andExpect(StreamResultMatcher.expectOctetStream())
            .andExpect(StreamResultMatcher.expectMinimumSize(50));
    }
    
    @Test
    public void testNotFound() throws Exception {
    	String exerciseId = "1"; // warning: this depends on the sequence in the current search index :-(
    	restExerciseMockMvc.perform(get("/api/exerciseFile/"+exerciseId).param("filePath", "fileDoesNotExist.md")
            	.with(csrf().asHeader())
            .contentType(MediaType.APPLICATION_JSON))
    		.andDo(print())
    		.andExpect(status().isNotFound());

    }
    
    public static class StreamResultMatcher {
    	
    	public static ResultMatcher expectMinimumSize(int size) {
    		return result -> {
    			final byte[] bytes = result.getResponse().getContentAsByteArray();
    			Assert.assertTrue("minimum size should be  " + size + ", but was " + bytes.length,
    					bytes.length >= size);
    		};
    	}
    	
    	public static ResultMatcher expectOctetStream() {
    		return result -> {
    			Assert.assertEquals("Expected octet stream", MediaType.APPLICATION_OCTET_STREAM.toString(), result.getResponse().getContentType());
    		};
    	}
    }
    
    @Test
    public void getExerciseById() {
    	
    }

}
